import os


class FileOrDirectory:
    @property
    def name(self):
        return self.path.split('/')[-1]

    @property
    def exists(self):
        return os.path.exists(self.path)

    def __eq__(self, other):
        print(self.path, other.path)
        if isinstance(self, type(other)):
            return self.path == other.path
        return False



"""Utils for reading remote files."""
import json
import ssl

import requests
from selenium import webdriver
from selenium.webdriver.firefox.options import Options

from utils.File import File, XSVFile

USER_AGENT = (
    'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.14; rv:65.0) '
    + 'Gecko/20100101 Firefox/65.0'
)
ENCODING = 'utf-8'
SELENIUM_SCROLL_REPEATS = 3
SELENIUM_SCROLL_WAIT_TIME = 0.5
EXISTS_TIMEOUT = 1

# pylint: disable=W0212
ssl._create_default_https_context = ssl._create_unverified_context


class WWW:
    def __init__(self, url: str):
        self.url = url

    def readBinary(self):
        try:
            resp = requests.get(self.url, headers={'user-agent': USER_AGENT})
            if resp.status_code != 200:
                raise Exception(
                    f'Failed to read {self.url}. resp.status_code != 200. '
                )
            content = resp.content
            content_size = len(content)
            if content_size == 0:
                raise Exception(
                    f'Failed to read {self.url}. content_size == 0.'
                )
            return content
        except requests.exceptions.ConnectionError as e:
            raise Exception(f'Failed to read {self.url}. {e}')

    def readSelenium(self):
        options = Options()
        options.headless = True
        driver = webdriver.Firefox(options=options)
        driver.get(self.url)
        content = driver.page_source
        driver.quit()
        return content

    def read(self):
        return self.readBinary().decode()

    def readJSON(self):
        content = self.read()
        return json.loads(content) if content else None

    def readXSV(self, separator):
        content = self.read()
        return XSVFile._readHelper(separator, content.split('\n'))

    def readCSV(self):
        return self.readXSV(',')

    def readTSV(self):
        return self.readXSV('\t')

    def downloadBinary(self, file_name):
        content = self.readBinary()
        if content:
            File(file_name).writeBinary(content)

    @property
    def exists(self):
        try:
            response = requests.head(self.url, timeout=EXISTS_TIMEOUT)
            # pylint: disable=E1101
            return response.status_code == requests.codes.ok
        except requests.exceptions.ConnectTimeout:
            return False



from dataclasses import dataclass

import tweepy

from utils.Log import Log

log = Log('Twitter')


@dataclass
class TwitterBase:
    twtr_api_key: str
    twtr_api_secret_key: str
    twtr_access_token: str
    twtr_access_token_secret: str

    @property
    def api(self):
        try:
            auth = tweepy.OAuthHandler(
                self.twtr_api_key, self.twtr_api_secret_key
            )
            log.debug('Created Twitter API auth.')
            auth.set_access_token(
                self.twtr_access_token, self.twtr_access_token_secret
            )
            log.debug('Set Twitter Access Token.')
            return tweepy.API(auth)
        except Exception as e:
            log.error(f'Twitter API Setup failed: {e}')
            return None



import os

from utils.File import File
from utils.FileOrDirectory import FileOrDirectory


class Directory(FileOrDirectory):
    @staticmethod
    def isIgnore(path: str):
        for k in ['__pycache__', '.git']:
            if k in path:
                return True
        return False

    def __init__(self, path):
        self.path = path

    @property
    def name(self):
        return self.path.split('/')[-1]

    @property
    def children(self):
        _children = []
        for name in os.listdir(self.path):
            path = os.path.join(self.path, name)
            if Directory.isIgnore(path):
                continue

            if os.path.isdir(path):
                _children.append(Directory(path))
            else:
                _children.append(File(path))
        return sorted(_children, key=lambda x: x.name)



from functools import cached_property

from utils.File import JSONFile


class FiledVariable:
    def __init__(self, key: str, func_get):
        self.key = key
        self.func_get = func_get

    @property
    def file_path(self):
        return f'/tmp/{self.key}.json'

    @property
    def file(self):
        return JSONFile(self.file_path)

    def clear(self):
        self.file.delete()

    @cached_property
    def value(self):
        if self.file.exists:
            return self.file.read()
        else:
            value = self.func_get()
            self.file.write(value)
            return value



import csv
import json
import os
import zipfile

from utils.FileOrDirectory import FileOrDirectory

DIALECT = 'excel'
DELIMITER_CSV = ','
DELIMITER_TSV = '\t'
DELIM_LINE = '\n'


class File(FileOrDirectory):
    def __init__(self, path):
        self.path = path

    def delete(self):
        if self.exists:
            os.remove(self.path)

    @property
    def ext(self):
        return self.name.split('.')[-1]

    def read(self):
        with open(self.path, 'r') as fin:
            content = fin.read()
            fin.close()
        return content

    def readBinary(self):
        with open(self.path, 'rb') as fin:
            content = fin.read()
            fin.close()
        return content

    def write(self, content):
        with open(self.path, 'w') as fout:
            fout.write(content)
            fout.close()

    def writeBinary(self, content):
        with open(self.path, 'wb') as fout:
            fout.write(content)
            fout.close()

    def read_lines(self):
        content = File.read(self)
        return content.split(DELIM_LINE)

    def write_lines(self, lines):
        content = DELIM_LINE.join(lines)
        File.write(self, content)


class JSONFile(File):
    def read(self):
        content = File.read(self)
        return json.loads(content)

    def write(self, data):
        content = json.dumps(data, indent=2)
        File.write(self, content)


class XSVFile(File):
    def __init__(self, path, delimiter):
        File.__init__(self, path)
        self.delimiter = delimiter

    @staticmethod
    def _readHelper(delimiter: str, xsv_lines: list):
        data_list = []
        field_names = None
        reader = csv.reader(
            xsv_lines,
            dialect=DIALECT,
            delimiter=delimiter,
        )
        for row in reader:
            if not field_names:
                field_names = row
            else:
                data = dict(
                    zip(
                        field_names,
                        row,
                    )
                )
                if data:
                    data_list.append(data)
        return data_list

    def read(self):
        xsv_lines = File.read_lines(self)
        return XSVFile._readHelper(self.delimiter, xsv_lines)

    @staticmethod
    def get_field_names(data_list):
        return list(data_list[0].keys())

    @staticmethod
    def get_data_rows(data_list, field_names):
        return list(
            map(
                lambda data: list(
                    map(
                        lambda field_name: data[field_name],
                        field_names,
                    )
                ),
                data_list,
            )
        )

    def write(self, data_list):
        with open(self.path, 'w') as fout:
            writer = csv.writer(
                fout,
                dialect=DIALECT,
                delimiter=self.delimiter,
            )

            field_names = XSVFile.get_field_names(data_list)
            writer.writerow(field_names)
            writer.writerows(XSVFile.get_data_rows(data_list, field_names))
            fout.close()


class CSVFile(XSVFile):
    def __init__(self, path):
        return XSVFile.__init__(self, path, DELIMITER_CSV)


class TSVFile(XSVFile):
    def __init__(self, path):
        return XSVFile.__init__(self, path, DELIMITER_TSV)


class Zip:
    def __init__(self, path):
        self.path = path

    @property
    def zip_path(self):
        return self.path + '.zip'

    @property
    def arc_name(self):
        return os.path.basename(os.path.normpath(self.path))

    @property
    def dir_zip(self):
        return os.path.dirname(os.path.normpath(self.path))

    def zip(self, skip_delete=False):
        assert os.path.exists(self.path)
        with zipfile.ZipFile(
            self.zip_path,
            mode='w',
            compression=zipfile.ZIP_DEFLATED,
        ) as zip_file:
            zip_file.write(self.path, arcname=self.arc_name)
            assert os.path.exists(self.zip_path)

        if not skip_delete:
            os.remove(self.path)
            assert not os.path.exists(self.path)

    def unzip(self, skip_delete=False):
        assert os.path.exists(self.zip_path)
        with zipfile.ZipFile(self.zip_path) as zip_file:
            zip_file.extractall(self.dir_zip)
            assert os.path.exists(self.path)

        if not skip_delete:
            os.remove(self.zip_path)
            assert not os.path.exists(self.zip_path)



from utils.Browser import Browser
from utils.Color import Color
from utils.Dict import Dict
from utils.Directory import Directory
from utils.File import CSVFile, File, JSONFile, TSVFile, XSVFile, Zip
from utils.FiledVariable import FiledVariable
from utils.Git import Git
from utils.List import List
from utils.Log import Log
from utils.String import String
from utils.Table import Table, TableRow
from utils.time import (DAYS_IN, SECONDS_IN, TIME_FORMAT_DATE,
                        TIME_FORMAT_DATE_ID, TIME_FORMAT_TIME,
                        TIME_FORMAT_TIME_ID, TIMEZONE_OFFSET, Time, TimeDelta,
                        TimeFormat, TimeUnit, get_date_id, get_time_id)
from utils.twitter.Tweet import Tweet
from utils.twitter.Twitter import Twitter
from utils.www import WWW
